from pyfortimanager.core.fortimanager import FortiManager


class FortiAPs(FortiManager):
    """API class for FortiAPs.
    """

    def __init__(self, **kwargs):
        super(FortiAPs, self).__init__(**kwargs)

    def all(self, fortigate: str, vdom: str="root", wtp_id: str=None, adom: str=None):
        """Retrieves all FortiAPs or a single FortiAP from a FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            vdom (str): Name of the virtual domain for the FortiGate.
            wtp_id (str): Serial number of a specific FortiAP.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/wireless-controller/wtp",
            "scope member": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        if wtp_id:
            params['url'] += f"/{wtp_id}"

        return self.post(method="get", params=params)

    def upgrade(self, fortigate: str, wtp_id: str, image: str):
        """Updates the firmware of a specific FortiAP.

        Args:
            fortigate (str): Name of the FortiGate.
            wtp_id (str): Serial number of the FortiAP.
            image (str): Support version with a build number. e.g., 6.4.12-b2060.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/um/image/upgrade/ext",
            "data": {
                "create_task": "enable",
                "devices": [
                    {
                        "controllers": [
                            {
                                "id": wtp_id,
                                "image": image
                            }
                        ],
                        "name": fortigate
                    }
                ],
				"flags": 0
            }
        }

        return self.post(method="exec", params=params)

    def refresh(self, fortigates: list, adom: str=None):
        """Refreshes all FortiAPs from one or more FortiGates.

        Args:
            fortigates (list): List of FortiGate OID's to refresh. Example: [60123, 601234]
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/deployment/get/controller/status",
            "data": {
                "adom": adom or self.api.adom,
                "ctype": 1,
                "device": fortigates,
                "options": 3,
                "resync": 1
            }
        }

        return self.post(method="exec", params=params)

    def profiles(self, name: str=None, adom: str=None):
        """Retrieves a list of all AP profiles or a single AP profile.

        Args:
            name (str): Name of a specific AP profile.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/wireless-controller/wtp-profile"
        }

        if name:
            params['url'] += f"/{name}"

        return self.post(method="get", params=params)

    def add(self, name: str, wtp_id: str, fortigate: str, vdom: str="root", wtp_profile: str=None, prefer_img_ver: str=None, description: str=None, adom: str=None):
        """Adds a new FortiAP.

        Args:
            name (str): Name of the FortiAP.
            wtp_id (str): Serial number of the FortiAP.
            fortigate (str): Name of the FortiGate connected to the FortiSwitch.
            vdom (str): Name of the virtual domain for the FortiGate.
            wtp_profile (str, optional): Name of the AP profile used for this AP.
            prefer_img_ver (str, optional): Enforce the firmware version of the FortiAP. Ex. 7.2.3-b0365.
            description (str, optional): Description of the FortiAP.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/wireless-controller/wtp",
            "data": {
                "name": name,
                "wtp-id": wtp_id
            },
            "scope member": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        # Optional fields
        if wtp_profile:
            params['data']['wtp-profile'] = wtp_profile
        
        if prefer_img_ver:
            params['data']['_prefer-img-ver'] = prefer_img_ver

        if description:
            params['data']['location'] = description

        return self.post(method="add", params=params)

    def update(self, wtp_id: str, fortigate: str, vdom: str="root", name: str=None, wtp_profile: str=None, description: str=None, adom: str=None):
        """Updates a FortiAP.

        Args:
            wtp_id (str): Serial number of the FortiAP to update.
            fortigate (str): Name of the FortiGate connected to the FortiSwitch.
            vdom (str): Name of the virtual domain for the FortiGate.
            name (str, optional): Name of the FortiAP.
            wtp_profile (str, optional): Name of the AP profile used for this AP.
            description (str, optional): Description of the FortiAP.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/wireless-controller/wtp",
            "data": {
                "wtp-id": wtp_id
            },
            "scope member": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        # Optional fields
        if name:
            params['data']['name'] = name

        if wtp_profile:
            params['data']['wtp-profile'] = wtp_profile

        if description:
            params['data']['location'] = description

        return self.post(method="update", params=params)

    def delete(self, wtp_id: str, fortigate: str, vdom: str="root", adom: str=None):
        """Deletes a FortiAP.

        Args:
            wtp_id (str): Serial number of the FortiAP to delete.
            fortigate (str): Name of the FortiGate connected to the FortiSwitch.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/wireless-controller/wtp/{wtp_id}",
            "scope member": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }
        
        return self.post(method="delete", params=params)


from pyfortimanager.models.adoms import ADOMs
from pyfortimanager.models.cli_template_groups import CLI_Template_Groups
from pyfortimanager.models.device_groups import Device_Groups
from pyfortimanager.models.fortiaps_proxy import FortiAPs_Proxy
from pyfortimanager.models.fortiaps import FortiAPs
from pyfortimanager.models.fortigates_proxy import FortiGates_Proxy
from pyfortimanager.models.fortigates import FortiGates
from pyfortimanager.models.fortiswitches_proxy import FortiSwitches_Proxy
from pyfortimanager.models.fortiswitches import FortiSwitches
from pyfortimanager.models.install_wizard import Install_Wizard
from pyfortimanager.models.policy_packages import Policy_Packages
from pyfortimanager.models.radius_servers import RADIUS_Servers
from pyfortimanager.models.scripts import Scripts
from pyfortimanager.models.sdwan_templates import SDWAN_Templates
from pyfortimanager.models.system import System


class Api(object):
    """Base API class.
    """

    def __init__(self, host: str, username: str, password: str, adom: str="root", verify: bool=True, proxy_timeout: int=60, **kwargs):
        self.host = host
        self.username = username
        self.password = password
        self.adom = adom
        self.verify = verify
        self.proxy_timeout = proxy_timeout
        self.session = None
        self.sessionid = None

    @property
    def adoms(self):
        """Endpoints related to ADOM management.
        """
        return ADOMs(api=self)

    @property
    def cli_template_groups(self):
        """Endpoints related to CLI Template Groups.
        """
        return CLI_Template_Groups(api=self)

    @property
    def device_groups(self):
        """Endpoints related to Device Groups.
        """
        return Device_Groups(api=self)

    @property
    def fortiaps_proxy(self):
        """Endpoints related to FortiAP proxy calls on a FortiGate.
        """
        return FortiAPs_Proxy(api=self)

    @property
    def fortiaps(self):
        """Endpoints related to FortiAP management.
        """
        return FortiAPs(api=self)
    
    @property
    def fortigates_proxy(self):
        """Endpoints related to proxy calls on a FortiGate.
        """
        return FortiGates_Proxy(api=self)

    @property
    def fortigates(self):
        """Endpoints related to FortiGate management.
        """
        return FortiGates(api=self)

    @property
    def fortiswitches_proxy(self):
        """Endpoints related to FortiSwitch proxy calls on a FortiGate.
        """
        return FortiSwitches_Proxy(api=self)

    @property
    def fortiswitches(self):
        """Endpoints related to FortiSwitch management.
        """
        return FortiSwitches(api=self)

    @property
    def install_wizard(self):
        """Endpoints related to the Install Wizard.
        """
        return Install_Wizard(api=self)

    @property
    def policy_packages(self):
        """Endpoints related to Policy Packages.
        """
        return Policy_Packages(api=self)

    @property
    def radius_servers(self):
        """Endpoints related to RADIUS_Servers.
        """
        return RADIUS_Servers(api=self)

    @property
    def scripts(self):
        """Endpoints related to Scripts.
        """
        return Scripts(api=self)

    @property
    def sdwan_templates(self):
        """Endpoints related to SD-WAN Templates.
        """
        return SDWAN_Templates(api=self)

    @property
    def system(self):
        """Endpoints related to the FortiManager system.
        """
        return System(api=self)


from pyfortimanager.core.fortimanager import FortiManager


class Install_Wizard(FortiManager):
    """API class for the Install Wizard.
    """

    def __init__(self, **kwargs):
        super(Install_Wizard, self).__init__(**kwargs)

    def device_settings(self, fortigate: str, vdom: str="root", adom: str=None):
        """Installs the device settings on a FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/securityconsole/install/device",
            "data": {
                "adom": adom or self.api.adom,
                "scope": [
                    {
                        "name": fortigate,
                        "vdom": vdom
                    }
                ],
            }
        }

        return self.post(method="exec", params=params)

    def policy_package(self, policy_package: str, fortigate: str, vdom: str="root", adom: str=None):
        """Installs a policy package on a FortiGate.

        Args:
            policy_package (str): Name of the policy package to be installed.
            fortigate (str): Name of the FortiGate.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/securityconsole/install/package",
            "data": {
                "adom": adom or self.api.adom,
                "flags": "nonblocking",
                "pkg": policy_package,
                "scope": [
                    {
                        "name": fortigate,
                        "vdom": vdom
                    }
                ],
            }
        }

        return self.post(method="exec", params=params)


from pyfortimanager.core.fortimanager import FortiManager


class SDWAN_Templates(FortiManager):
    """API class for SD-WAN Templates in the Device Manager.
    """

    def __init__(self, **kwargs):
        super(SDWAN_Templates, self).__init__(**kwargs)

    def all(self, name: str=None, adom: str=None):
        """Retrieves all SD-WAN templates or a single SD-WAN template with members.

        Args:
            name (str): Name of the SD-WAN template.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/wanprof/adom/{adom or self.api.adom}"
        }

        # Retrieve a single SD-WAN template
        if name:
            params['url'] += f"/{name}"

        return self.post(method="get", params=params)

    def delete(self, name: str, adom: str=None):
        """Deletes a SD-WAN template.

        Args:
            name (str): Name of the SD-WAN template to delete.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/wanprof/adom/{adom or self.api.adom}/{name}",
            "confirm": 1
        }

        return self.post(method="delete", params=params)

    def add_member(self, name: str, fortigate: str, vdom: str="root", adom: str=None):
        """Adds a FortiGate as a member to a SD-WAN template.

        Args:
            name (str): Name of the SD-WAN template.
            fortigate (str): Name of the FortiGate to add as a member.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/wanprof/adom/{adom or self.api.adom}/{name}/scope member",
            "data": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        return self.post(method="add", params=params)

    def remove_member(self, name: str, fortigate: str, vdom: str="root", adom: str=None):
        """Removes a FortiGate as a member from a SD-WAN template.

        Args:
            name (str): Name of the SD-WAN template.
            fortigate (str): Name of the FortiGate to remove as a member.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/wanprof/adom/{adom or self.api.adom}/{name}/scope member",
            "data": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        return self.post(method="delete", params=params)


from pyfortimanager.core.fortimanager import FortiManager


class RADIUS_Servers(FortiManager):
    """API class for the RADIUS Servers in Policy & Objects.
    """

    def __init__(self, **kwargs):
        super(RADIUS_Servers, self).__init__(**kwargs)

    def all(self, name: str=None, adom: str=None):
        """Retrieves all RADIUS servers or a single RADIUS server.

        Args:
            name (str): Name of the radius server.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/user/radius",
        }

        if name:
            params['url'] += f"/{name}"

        return self.post(method="get", params=params)

    def add_member(self, fortigate: str, fortigate_source_ip: str, fortigate_nas_ip: str, radius_server: str, radius_server_ip: str, radius_secret: str, radius_secondary_server_ip: str=None, radius_secondary_secret: str=None, vdom: str="root", adom: str=None):
        """Adds a FortiGate as a member to a RADIUS server.

        Args:
            radius_server (str): Name of the RADIUS server to add the member to.
            fortigate (str): Name of the FortiGate to add as a member.
            vdom (str): Name of the virtual domain for the FortiGate.
            fortigate_source_ip (str): Source IP for the FortiGate.
            fortigate_nas_ip (str): NAS IP for the FortiGate.
            radius_server_ip (str): IP of the primary RADIUS server.
            radius_secret (str): Secret for the primary RADIUS server.
            radius_secondary_server_ip (str, optional): IP of the secondary RADIUS server.
            radius_secondary_secret (str, optional): Secret for the secondary RADIUS server.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/user/radius/{radius_server}/dynamic_mapping",
            "data": {
                "_scope": [
                    {
                        "name": fortigate,
                        "vdom": vdom
                    }
                ],
                "server": radius_server_ip,
                "secret": radius_secret,
                "source-ip": fortigate_source_ip,
                "nas-ip": fortigate_nas_ip
            }
        }

        # Optional fields
        if radius_secondary_server_ip:
            params['data']['secondary-server'] = radius_secondary_server_ip

        if radius_secondary_secret:
            params['data']['server'] = radius_secondary_secret

        return self.post(method="add", params=params)
     
    def update_member(self, radius_server: str, fortigate: str, vdom: str="root", fortigate_source_ip: str=None, fortigate_nas_ip: str=None, radius_server_ip: str=None, radius_secret: str=None, radius_secondary_server_ip: str=None, radius_secondary_secret: str=None, adom: str=None):
        """Updates a FortiGate member on the RADIUS server.

        Args:
            radius_server (str): Name of the RADIUS server to update the member on.
            fortigate (str): Name of the FortiGate member to update.
            vdom (str): Name of the virtual domain for the FortiGate.
            fortigate_source_ip (str): Source IP for the FortiGate.
            fortigate_nas_ip (str): NAS IP for the FortiGate.
            radius_server_ip (str): IP of the primary RADIUS server.
            radius_secret (str): Secret for the primary RADIUS server.
            radius_secondary_server_ip (str): IP of the secondary RADIUS server.
            radius_secondary_secret (str): Secret for the secondary RADIUS server.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/user/radius/{radius_server}/dynamic_mapping",
            "data": {
                "_scope": [
                    {
                        "name": fortigate,
                        "vdom": vdom
                    }
                ]
            }
        }

        # Optional fields
        if radius_server_ip:
            params['data']['server'] = radius_server_ip

        if radius_secret:
            params['data']['secret'] = radius_secret

        if radius_secondary_server_ip:
            params['data']['secondary-server'] = radius_secondary_server_ip

        if radius_secondary_secret:
            params['data']['server'] = radius_secondary_secret

        if fortigate_source_ip:
            params['data']['source-ip'] = fortigate_source_ip

        if fortigate_nas_ip:
            params['data']['nas-ip'] = fortigate_nas_ip

        return self.post(method="update", params=params)


from pyfortimanager.core.fortimanager import FortiManager


class Device_Groups(FortiManager):
    """API class for Device Groups in the Device Manager.
    """

    def __init__(self, **kwargs):
        super(Device_Groups, self).__init__(**kwargs)

    def all(self, name: str=None, adom: str=None):
        """Retrieves all device groups or a single device group with members.

        Args:
            name (str): Name of a specific device group.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{adom or self.api.adom}/group",
            "option": [
                "object member"
            ],
        }

        # Retrieve a specific device group
        if name:
            params['url'] += f"/{name}"

        return self.post(method="get", params=params)

    def add(self, name: str, description:str=None, adom: str=None):
        """Adds a device group.

        Args:
            name (str): Name of the device group.
            description (str, optional): Description of the device group.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{adom or self.api.adom}/group/{name}",
            "data": {
				"name": name,
				"desc": description
            }
        }

        return self.post(method="add", params=params)
    
    def update(self, name: str, rename: str=None, description:str=None, adom: str=None):
        """Updates a device group.

        Args:
            name (str): Name of the device group to update.
            rename (str, optional): New name of the device group.
            description (str, optional): Description of the device group.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{adom or self.api.adom}/group/{name}",
            "data": {}
        }

        # Optional fields
        if rename:
            params['data']['name'] = rename

        if description:
            params['data']['desc'] = description

        return self.post(method="update", params=params)

    def delete(self, name: str, adom: str=None):
        """Deletes a devoce group.

        Args:
            name (str): Name of the device group to delete.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{adom or self.api.adom}/group/{name}"
        }

        return self.post(method="delete", params=params)

    def add_member(self, name: str, fortigate: str, vdom: str="root", adom: str=None):
        """Adds a FortiGate as a member to a device group.

        Args:
            name (str): Name of the device group.
            fortigate (str): Name of the FortiGate to add as a member.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{adom or self.api.adom}/group/{name}/object member",
            "data": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        return self.post(method="add", params=params)

    def remove_member(self, name: str, fortigate: str, vdom: str="root", adom: str=None):
        """Removes a FortiGate as a member from a device group.

        Args:
            name (str): Name of the device group.
            fortigate (str): Name of the FortiGate to remove as a member.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{adom or self.api.adom}/group/{name}/object member",
            "data": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        return self.post(method="delete", params=params)


from requests.adapters import HTTPAdapter, Retry
import requests
import atexit


class FortiManager(object):
    """API class for FortiManager login management and post requests.
    """

    def __init__(self, api, **kwargs):
        self.api = api
        self.base_url = f"{self.api.host}/jsonrpc"
        self.max_retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
        atexit.register(self.logout)

    def login(self):
        """Logs in to the FortiManager API.

        Returns:
            dict: JSON data.
        """

        # Create a new session
        self.api.session = requests.session()

        data = {
            "method": "exec",
            "params": [
                {
                    "data": {
                        "passwd": self.api.password,
                        "user": self.api.username
                    },
                    "url": "/sys/login/user"
                }
            ],
            "session": self.api.sessionid
        }

        self.api.session.mount("http://", HTTPAdapter(max_retries=self.max_retries))
        response = self.api.session.post(url=self.base_url, json=data, verify=self.api.verify)

        # HTTP 200 OK
        if response.status_code == 200:

            # Check if the FortiManager request is successful
            if response.json()['result'][0]['status']['code'] == 0:
                self.api.sessionid = response.json()['session']

            return response.json()

    def logout(self):
        """Logs out of the FortiManager API.

        Returns:
            dict: JSON data.
        """

        # Only try the log out endpoint, if we have a session.
        if self.api.session:
            data = {
                "method": "exec",
                "params": [
                    {
                        "url": "/sys/logout"
                    }
                ],
                "session": self.api.sessionid
            }

            self.api.session.mount("http://", HTTPAdapter(max_retries=self.max_retries))
            response = self.api.session.post(url=self.base_url, json=data, verify=self.api.verify)
            
            # HTTP 200 OK
            if response.status_code == 200:
                return response.json()

        # Clear our session variables
        self.api.session = None
        self.api.sessionid = None

    def login_check(self):
        """Checks if we have a valid login session.
        """

        # Check if we have a session
        if self.api.session:
          
            # Use the sys_status endpoint as a test to see if our session is valid in FortiManager
            data = {
                "method": "get",
                "params": [
                    {
                        "url": "/sys/status"
                    }
                ],
                "session": self.api.sessionid
            }

            self.api.session.mount("http://", HTTPAdapter(max_retries=self.max_retries))
            sys_status = self.api.session.post(url=self.base_url, json=data, verify=self.api.verify)

            # HTTP 200 OK
            if sys_status.status_code == 200:
              
                # If the FortiManager request is unsuccessful, log in again.
                if sys_status.json()['result'][0]['status']['code'] != 0:
                    self.login()
                  
            # Log in again.
            else:
              self.login()
                  
        # Log in again.
        else:
            self.login()

    def post(self, method: str, params: dict):
        """Sends a POST request to the FortiManager API.

        Args:
            method (str): get, exec, add, set, update, delete.
            params (dict): Payload data to send with the request.

        Returns:
            dict: JSON data.
        """

        self.login_check()

        data = {
            "method": method,
            "params": [params],
            "session": self.api.sessionid
        }

        self.api.session.mount("http://", HTTPAdapter(max_retries=self.max_retries))
        response = self.api.session.post(url=self.base_url, json=data, verify=self.api.verify)

        # HTTP 200 OK
        if response.status_code == 200:
            return response.json()['result'][0]


from pyfortimanager.core.fortimanager import FortiManager


class ADOMs(FortiManager):
    """API class for ADOM management.
    """

    def __init__(self, **kwargs):
        super(ADOMs, self).__init__(**kwargs)

    def all(self, name: str=None):
        """Retrieves all ADOMs or a single ADOM.

        Args:
            name (str): Name of a ADOM.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/dvmdb/adom"
        }

        # Retrieve a specific ADOM
        if name:
            params['url'] += f"/{name}"

        return self.post(method="get", params=params)

    def lock(self, name: str):
        """Locks an ADOM.

        Args:
            name (str): Name of the ADOM to lock.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{name}/workspace/lock"
        }

        return self.post(method="exec", params=params)
    
    def unlock(self, name: str):
        """Unlocks an ADOM.

        Args:
            name (str): Name of the ADOM to unlock.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{name}/workspace/unlock"
        }

        return self.post(method="exec", params=params)
    
    def commit(self, name: str):
        """Commit changes to an entire ADOM.

        Args:
            name (str): Name of the ADOM to commit changes in.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{name}/workspace/commit"
        }

        return self.post(method="exec", params=params)


from pyfortimanager.core.fortimanager import FortiManager


class FortiGates_Proxy(FortiManager):
    """API class for proxy calls to FortiGates. Send and receive API calls to/from an online FortiGate.
    """

    def __init__(self, **kwargs):
        super(FortiGates_Proxy, self).__init__(**kwargs)

    def status(self, fortigate: str, adom: str=None, timeout: int=None):
        """Retrieve basic system status on the FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.
            timeout (int, optional): How long to wait for the FortiGate to respond. Defaults to the proxy_timeout set when the API was instantiated.
            
        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/sys/proxy/json",
            "data":
                {
                    "target": [
                        f"/adom/{adom or self.api.adom}/device/{fortigate}"
                    ],
                    "action": "get",
                    "timeout": timeout or self.api.proxy_timeout,
                    "resource": f"/api/v2/monitor/system/status"
                }
        }

        return self.post(method="exec", params=params)

    def resource_usage(self, fortigate: str, adom: str=None, scope: str="global", resource: str=None, interval: str=None, timeout: int=None):
        """Retrieves current and historical usage data for a provided resource on the FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.
            scope (str): Scope from which to retrieve the interface stats from [vdom|global]. Default is global.
            resource (str. optional): Get a specific resource to get usage data for. Defaults to all resources. 
            interval (str, optional): Time interval of resource usage [1-min|10-min|30-min|1-hour|12-hour|24-hour]. Defaults to all intervals.
            timeout (int, optional): How long to wait for the FortiGate to respond. Defaults to the proxy_timeout set when the API was instantiated.
            
        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/sys/proxy/json",
            "data":
                {
                    "target": [
                        f"/adom/{adom or self.api.adom}/device/{fortigate}"
                    ],
                    "action": "get",
                    "timeout": timeout or self.api.proxy_timeout,
                    "resource": f"/api/v2/monitor/system/resource/usage/?scope={scope}"
                }
        }

        # Optional fields
        if resource:
            params['data']['resource'] += f"&resource={resource}"

        if interval:
            params['data']['resource'] += f"&interval={interval}"

        return self.post(method="exec", params=params)

    def interfaces(self, fortigate: str, adom: str=None, scope: str="global", include_vlan: bool=True, include_aggregate: bool=True, interface: str=None, timeout: int=None):
        """Retrieves a list of interfaces or a specific interface and their configuration on the FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.
            scope (str): Scope from which to retrieve the interface stats from [vdom|global]. Default is global.
            include_vlan (bool): Enable to include VLANs in result list. Default is True.
            include_aggregate (bool): Enable to include Aggregate interfaces in result list. Default is True.
            interface (str, optional): Retrieve this interface only.
            timeout (int, optional): How long to wait for the FortiGate to respond. Defaults to the proxy_timeout set when the API was instantiated.
            
        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/sys/proxy/json",
            "data":
                {
                    "target": [
                        f"/adom/{adom or self.api.adom}/device/{fortigate}"
                    ],
                    "action": "get",
                    "timeout": timeout or self.api.proxy_timeout,
                    "resource": f"/api/v2/monitor/system/interface/?scope={scope}&include_vlan={include_vlan}&include_aggregate={include_aggregate}"
                }
        }

        # Optional fields
        if interface:
            params['data']['resource'] += f"&interface_name={interface}"

        return self.post(method="exec", params=params)

    def transceivers(self, fortigate: str, adom: str=None, scope: str="global", timeout: int=None):
        """Retrieves a list of transceivers used on the FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.
            scope (str): Scope from which to retrieve the interface stats from [vdom|global]. Default is global.
            timeout (int, optional): How long to wait for the FortiGate to respond. Defaults to the proxy_timeout set when the API was instantiated.
            
        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/sys/proxy/json",
            "data":
                {
                    "target": [
                        f"/adom/{adom or self.api.adom}/device/{fortigate}"
                    ],
                    "action": "get",
                    "timeout": timeout or self.api.proxy_timeout,
                    "resource": f"/api/v2/monitor/system/interface/transceivers?scope={scope}"
                }
        }

        return self.post(method="exec", params=params)

    def dhcp_leases(self, fortigate: str, adom: str=None, scope: str="global", ipv6: bool=True, interface: str=None, timeout: int=None):
        """Retrieves a list of all DHCP and DHCPv6 leases on the FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.
            scope (str): Scope from which to retrieve the interface stats from [vdom|global]. Default is global.
            ipv6 (bool): Include IPv6 addresses in the response. Default is True.
            interface (str, optional): Retrieve DHCP leases for this interface only.
            timeout (int, optional): How long to wait for the FortiGate to respond. Defaults to the proxy_timeout set when the API was instantiated.
            
        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/sys/proxy/json",
            "data":
                {
                    "target": [
                        f"/adom/{adom or self.api.adom}/device/{fortigate}"
                    ],
                    "action": "get",
                    "timeout": timeout or self.api.proxy_timeout,
                    "resource": f"/api/v2/monitor/system/dhcp/?scope={scope}&ipv6={ipv6}"
                }
        }

        # Optional fields
        if interface:
            params['data']['resource'] += f"&interface={interface}"

        return self.post(method="exec", params=params)


from pyfortimanager.core.fortimanager import FortiManager


class Policy_Packages(FortiManager):
    """API class for Policy Packages in Policy & Objects.
    """

    def __init__(self, **kwargs):
        super(Policy_Packages, self).__init__(**kwargs)

    def all(self, name: str=None, adom: str=None):
        """Retrieves all policy packages or a single policy package with members.

        Args:
            name (str, optional): Name of a policy package.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/pkg/adom/{adom or self.api.adom}"
        }

        # Retrieve a single policy package
        if name:
            params['url'] += f"/{name}"

        return self.post(method="get", params=params)

    def add_member(self, name: str, fortigate: str, vdom: str="root", adom: str=None):
        """Adds a FortiGate as a member to a policy package.

        Args:
            name (str): Name of the policy package.
            fortigate (str): Name of the FortiGate to be added as a member.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/pkg/adom/{adom or self.api.adom}/{name}/scope member",
            "data": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        return self.post(method="add", params=params)

    def remove_member(self, name: str, fortigate: str, vdom: str="root", adom: str=None):
        """Removes a FortiGate as a member from a policy package.

        Args:
            name (str): Name of the policy package.
            fortigate (str): Name of the FortiGate to remove as a member.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/pkg/adom/{adom or self.api.adom}/{name}/scope member",
            "data": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        return self.post(method="delete", params=params)


from pyfortimanager.core.fortimanager import FortiManager


class FortiAPs_Proxy(FortiManager):
    """API class for proxy calls to FortiAPs. Send and receive API calls to/from an online FortiGate.
    """

    def __init__(self, **kwargs):
        super(FortiAPs_Proxy, self).__init__(**kwargs)

    def all(self, fortigate: str, adom: str=None, timeout: int=None):
        """Retrieves a list of managed FortiAPs on the FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.
            timeout (int, optional): How long to wait for the FortiGate to respond. Defaults to the proxy_timeout set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/sys/proxy/json",
            "data":
                {
                    "target": [
                        f"/adom/{adom or self.api.adom}/device/{fortigate}"
                    ],
                    "action": "get",
                    "timeout": timeout or self.api.proxy_timeout,
                    "resource": "/api/v2/monitor/wifi/managed_ap"
                }
        }

        return self.post(method="exec", params=params)

    def authorize(self, wtp_id: str, fortigate: str, vdom: str="root", adom: str=None, timeout: int=None):
        """Authorizes a FortiAP on the FortiGate.

        Args:
            wtp_id (str): Serial number of the FortiAP to authorize.
            fortigate (str): Name of the FortiGate.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.
            timeout (int, optional): How long to wait for the FortiGate to respond. Defaults to the proxy_timeout set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/sys/proxy/json",
            "data":
                {
                    "target": [
                        f"/adom/{adom or self.api.adom}/device/{fortigate}"
                    ],
                    "action": "post",
                    "payload": {
                        "vdom": vdom,
                        "wtpname": wtp_id,
                        "admin": "enable"
                    },
                    "timeout": timeout or self.api.proxy_timeout,
                    "resource": "/api/v2/monitor/wifi/managed_ap/set_status"
                }
        }

        return self.post(method="exec", params=params)
    
    def clients(self, fortigate: str, vdom: str="root", adom: str=None, timeout: int=None):
        """Retrieves a list of all connected Wi-Fi clients on the FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.
            timeout (int, optional): How long to wait for the FortiGate to respond. Defaults to the proxy_timeout set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/sys/proxy/json",
            "data":
                {
                    "target": [
                        f"/adom/{adom or self.api.adom}/device/{fortigate}"
                    ],
                    "action": "get",
                    "timeout": timeout or self.api.proxy_timeout,
                    "resource": f"/api/v2/monitor/wifi/client?vdom={vdom}"
                }
        }

        return self.post(method="exec", params=params)


from pyfortimanager.core.fortimanager import FortiManager


class FortiSwitches(FortiManager):
    """API class for FortiSwitches.
    """

    def __init__(self, **kwargs):
        super(FortiSwitches, self).__init__(**kwargs)

    def all(self, fortigate: str, vdom: str="root", switch_id: str=None, adom: str=None):
        """Retrieves all FortiSwitches or a single FortiSwitch from a FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            vdom (str): Name of the virtual domain for the FortiGate.
            switch_id (str, optional): Serial number of a specific FortiSwitch.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/fsp/managed-switch",
            "scope member": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        # Retrieve a single FortiSwitch
        if switch_id:
            params['url'] += f"/{switch_id}"

        return self.post(method="get", params=params)

    def upgrade(self, fortigate: str, switch_id: str, image: str):
        """Updates the firmware of a specific FortiSwitch.

        Args:
            fortigate (str): Name of the FortiGate.
            switch_id (str): Serial number of the FortiSwitch.
            image (str): Support version with a build number. e.g., 6.4.12-b2060.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/um/image/upgrade/ext",
            "data": {
                "create_task": "enable",
                "devices": [
                    {
                        "controllers": [
                            {
                                "id": switch_id,
                                "image": image
                            }
                        ],
                        "name": fortigate
                    }
                ],
				"flags": 0
            }
        }

        return self.post(method="exec", params=params)
    
    def refresh(self, fortigates: list, adom: str=None):
        """Refreshes all FortiSwitches from one or more FortiGates.

        Args:
            fortigates (list): List of FortiGate OID's to refresh. Example: [60123, 601234]
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/deployment/get/controller/status",
            "data": {
                "adom": adom or self.api.adom,
                "ctype": 4,
                "device": fortigates,
                "options": 3,
                "resync": 1
            }
        }

        return self.post(method="exec", params=params)

    def interfaces(self, switch_id: str, fortigate: str, vdom: str="root"):
        """Retrives all interfaces on the specified FortiSwitch.

        Args:
            switch_id (str): Serial number of the FortiSwitch.
            fortigate (str): Name of the FortiGate connected to the FortiSwitch.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/device/{fortigate}/vdom/{vdom}/switch-controller/managed-switch/{switch_id}"
        }

        return self.post(method="get", params=params)

    def interfaces_update(self, ports: list, switch_id: str, fortigate: str, vdom: str="root"):
        """Updates all interfaces on the specified FortiSwitch.

        Args:
            ports (list): List with all ports and their configuration.
            switch_id (str): Serial number of the FortiSwitch.
            fortigate (str): Name of the FortiGate connected to the FortiSwitch.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/device/{fortigate}/vdom/{vdom}/switch-controller/managed-switch/{switch_id}",
            "data": {
                "ports": ports
            }
        }

        return self.post(method="update", params=params)
    
    def add_to_adom(self, name: str, platform: str, switch_id: str, fortigate: str, vdom: str="root", interface: str="Fortilink", adom: str=None, prefer_img_ver: str=None):
        """Adds a new FortiSwitch as a model device in the FortiSwitch Manager (ADOM).

        Args:
            name (str): Name of the FortiSwitch.
            platform (str): Model name of the FortiSwitch. Ex. FortiSwitch-108E-FPOE.
            switch_id (str): Serial number of the FortiSwitch.
            interface (str): Name of the FortiGate interface connected to the FortiSwitch.
            fortigate (str): Name of the FortiGate connected to the FortiSwitch.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.
            prefer_img_ver (str, optional): Enforce the firmware version of the FortiSwitch. Ex. 7.2.5-b0453.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/fsp/managed-switch",
            "data": {
                "name": name,
                "platform": platform,
                "switch-id": switch_id,
                "vlan-interface": interface
            },
            "scope member": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        # Optional fields
        if prefer_img_ver:
            params['data']['prefer-img-ver'] = prefer_img_ver

        return self.post(method="add", params=params)
    
    def add_to_fortigate(self, name: str, switch_id: str, fortigate: str, vdom: str="root", prefer_img_ver: str=None):
        """Adds a new FortiSwitch as a model device on the FortiGate in the FortiSwitch Manager.

        Args:
            name (str): Name of the FortiSwitch.
            switch_id (str): Serial number of the FortiSwitch.
            fortigate (str): Name of the FortiGate connected to the FortiSwitch.
            vdom (str): Name of the virtual domain for the FortiGate.
            prefer_img_ver (str, optional): Enforce the firmware version for the FortiSwitch. Ex. 7.2.5-b0453.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/device/{fortigate}/vdom/{vdom}/switch-controller/managed-switch",
            "data": {
                "name": name,
                "switch-id": switch_id,
            }
        }

        # Optional fields
        if prefer_img_ver:
            params['data']['prefer-img-ver'] = prefer_img_ver
        
        return self.post(method="add", params=params)
    
    def update_in_adom(self, switch_id: str, fortigate: str, vdom: str="root", adom: str=None, name: str=None, prefer_img_ver: str=None):
        """Updates a FortiSwitch in the FortiSwitch Manager (ADOM).

        Args:
            switch_id (str): Serial number of the FortiSwitch to update.
            fortigate (str): Name of the FortiGate connected to the FortiSwitch.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.
            name (str, optional): Name of the FortiSwitch.
            prefer_img_ver (str, optional): Enforce the firmware version for the FortiSwitch. Ex. 7.2.5-b0453.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/fsp/managed-switch/{switch_id}",
            "data": {},
            "scope member": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ]
        }

        # Optional fields
        if name:
            params['data']['name'] = name

        if prefer_img_ver:
            params['data']['prefer-img-ver'] = prefer_img_ver
        
        return self.post(method="update", params=params)

    def update_on_fortigate(self, switch_id: str, fortigate: str, vdom: str="root", name: str=None, description: str=None):
        """Updates a FortiSwitch on the FortiGate in the FortiSwitch Manager.

        Args:
            switch_id (str): Serial number of the FortiSwitch to update.
            fortigate (str): Name of the FortiGate connected to the FortiSwitch.
            vdom (str): Name of the virtual domain for the FortiGate.
            name (str, optional): Name of the FortiSwitch.
            description (str, optional): Description of the FortiSwitch.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/device/{fortigate}/vdom/{vdom}/switch-controller/managed-switch/{switch_id}",
            "data": {}
        }

        # Optional fields
        if name:
            params['data']['name'] = name

        if description:
            params['data']['description'] = description

        return self.post(method="update", params=params)
    
    def delete_in_adom(self, switch_id: str, fortigate: str, vdom: str="root", adom: str=None):
        """Updates a FortiSwitch in the FortiSwitch Manager (ADOM).

        Args:
            switch_id (str): Serial number of the FortiSwitch to delete.
            fortigate (str): Name of the FortiGate.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/adom/{adom or self.api.adom}/obj/fsp/managed-switch/{switch_id}",
            "scope member": [
                {
                    "name": fortigate,
                    "vdom": vdom
                }
            ],
        }

        return self.post(method="delete", params=params)

    def delete_on_fortigate(self, switch_id: str, fortigate: str, vdom: str="root"):
        """Deletes a FortiSwitch on the FortiGate in the FortiSwitch Manager.

        Args:
            switch_id (str): Serial number of the FortiSwitch to delete.
            fortigate (str): Name of the FortiGate.
            vdom (str): Name of the virtual domain for the FortiGate.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/device/{fortigate}/vdom/{vdom}/switch-controller/managed-switch/{switch_id}"
        }

        return self.post(method="delete", params=params)


from pyfortimanager.core.fortimanager import FortiManager


class FortiGates(FortiManager):
    """API class for FortiGates.
    """

    def __init__(self, **kwargs):
        super(FortiGates, self).__init__(**kwargs)

    def all(self, fortigate: str=None, adom: str=None):
        """Retrieves all FortiGates or a single FortiGate.

        Args:
            name (str): Name of a specific FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{adom or self.api.adom}/device",
            "option": [
                "get meta"
            ]
        }

        # Retrieve a single FortiGate
        if fortigate:
            params['url'] += f"/{fortigate}"

        return self.post(method="get", params=params)

    def upgrade(self, fortigate: str, image: str):
        """Upgrades the firmware on the FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.
            image (str): Support version with a build number. e.g., 6.4.12-b2060.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/um/image/upgrade/ext",
            "data": {
                "adom": self.api.adom,
                "create_task": "enable",
				"devices": [
					{
						"image": image,
						"name": fortigate
					}
				],
				"flags": 0
            }
        }

        return self.post(method="exec", params=params)

    def refresh(self, fortigates: dict, adom: str=None):
        """Refreshes a list of FortiGates.

        Args:
            fortigates (dict): Dict of FortiGate name and OID's to refresh. Ex: { "name": "FortiGate-VM64-1", "oid": "12345" }, { "name": "FortiGate-VM64-2", "oid": "23456" }
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/dvm/cmd/update/dev-list",
            "data": {
                "adom": adom or self.api.adom,
                "flags": [
                    "create_task",
                    "nonblocking"
                ],
                "update-dev-member-list": [fortigates]
            }
        }

        return self.post(method="exec", params=params)

    def interfaces(self, fortigate: str, interface: str=None):
        """Retrieves all interfaces or a single interface from a FortiGate.

        Args:
            name (str): Name of the FortiGate.
            interface (str): Name of an interface.
    
        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/pm/config/device/{fortigate}/global/system/interface"
        }

        # Retrieve a single interface
        if interface:
            params['url'] = f"/{interface}"

        return self.post(method="get", params=params)
    
    def add(self, serial: str, mr: int, os_ver: int, name: str=None, mgmt_mode: str="fmg", os_type: str="fos", adm_usr: str=None, adm_pass: str=None, description: str=None, meta_fields: dict=None, flags:int=67371040, prefer_img_ver: str=None, adom: str=None):
        """Adds a new FortiGate as a model device in FortiManager.

        Args:
            name (str): Name of the FortiGate. Default is serial number.
            serial (str): Serial number of the FortiGate.
            mr (int): Minor OS version.
            os_ver (int): Major OS version.
            os_type (str): fos, fsw, foc, fml, faz, fwb, fch, fct, log, fmg, fsa, fdd, fac, fpx, fna, ffw, fsr, fad, fdc, fap, fxt, fts, fai, fwc, fis, fed. Default is fos.
            mgmt_mode (str): unreg, fmg, faz, fmgfaz. Default is fmg.
            flags (int): Various settings in the "Add Device"-dialog in FortiManager. Use FortiManager to retrieve a specific combination. Default is 67371040.
            adm_usr (str, optional): Default admin username.
            adm_pass (str, optional): Default admin password.
            description (str, optional): Description of the FortiGate.
            meta_fields (dict, optional): Meta fields for the FortiGate. Default is fmg.
            prefer_img_ver (str, optional): Enforce the firmware version of the FortiGate. Ex. 7.0.9-b444.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/dvm/cmd/add/device",
            "data": {
                "adom": adom or self.api.adom,
                "device": {
                    "flags": flags,
                    "mgmt_mode": mgmt_mode,
                    "mr": mr,
                    "name": name or serial,
                    "os_type": os_type,
                    "os_ver": os_ver,
                    "sn": serial
                }
            }
        }

        # Optional fields
        if adm_usr:
            params['data']['device']['adm_usr'] = adm_usr

        if adm_pass:
            params['data']['device']['adm_pass'] = adm_pass

        if description:
            params['data']['device']['desc'] = description

        if meta_fields:
            params['data']['device']['meta fields'] = meta_fields

        if prefer_img_ver:
            params['data']['device']['prefer_img_ver'] = prefer_img_ver

        return self.post(method="exec", params=params)
    
    def update(self, fortigate: str, meta_fields: dict=None, adm_pass: str=None, adm_usr: str=None, description: str=None, ip: str=None, latitude: float=None, longitude: float=None, name: str=None, hostname: str=None, prefer_img_ver: str=None, adom: str=None):
        """Updates a FortiGate.

        Args:
            fortigate (str): Name of the FortiGate to update.
            meta_fields (dict): All meta fields in a dict.
            adm_pass (str, optional): Admin password.
            adm_usr (str, optional): Admin username.
            description (str, optional): Description of the FortiGate.
            ip (str, optional): Public IP address of the FortiGate.
            latitude (float, optional): GPS latitude coordinates.
            longitude (float, optional): GPS longitude coordinates.
            name (str, optional): Name of the FortiGate.
            hostname (str, optional): Hostname of the FortiGate.
            prefer_img_ver (str, optional): Enforce the firmware version of the FortiGate. Ex. 7.0.9-b444.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{adom or self.api.adom}/device/{fortigate}",
            "data": {}
        }

        # Update optional fields
        if adm_pass:
            params['data']['adm_pass'] = adm_pass

        if adm_usr:
            params['data']['adm_usr'] = adm_usr

        if description:
            params['data']['desc'] = description

        if ip:
            params['data']['ip'] = ip

        if latitude:
            params['data']['latitude'] = latitude

        if longitude:
            params['data']['longitude'] = longitude

        if name:
            params['data']['name'] = name

        if hostname:
            params['data']['hostname'] = hostname

        if meta_fields:
            params['data']['meta fields'] = meta_fields

        if prefer_img_ver:
            params['data']['device']['prefer_img_ver'] = prefer_img_ver

        return self.post(method="update", params=params)
    
    def delete(self, fortigate: str, adom:str=None):
        """Deletes a FortiGate.

        Args:
            fortigate (str): Name of the FortiGate to delete.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/dvm/cmd/del/device",
            "data": {
                "adom": adom or self.api.adom,
                "device": fortigate
            }
        }

        return self.post(method="exec", params=params)


from pyfortimanager.core.fortimanager import FortiManager


class Scripts(FortiManager):
    """API class for Scripts in the Device Manager.
    """

    def __init__(self, **kwargs):
        super(Scripts, self).__init__(**kwargs)

    def all(self, name: str=None, adom: str=None):
        """Retrieves a list of all scripts or a single script.

        Args:
            name (str): Name of a script.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{adom or self.api.adom}/script"
        }

        # Retrieve a single script
        if name:
            params['url'] += f"/{name}"

        return self.post(method="get", params=params)

    def execute(self, name: str, fortigate: str, vdom: str="root", adom: str=None):
        """Executes a script on a FortiGate.

        Args:
            name (str): Name of the script to execute.
            fortigate (str): Name of the FortiGate to run the script on.
            vdom (str): Name of the virtual domain for the FortiGate.
            adom (str): Name of the ADOM. Defaults to the ADOM set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/adom/{adom or self.api.adom}/script/execute",
            "data": {
                "adom": self.api.adom,
                "script": name,
                "scope": [
                    {
                        "name": fortigate,
                        "vdom": vdom
                    }
                ],
            }
        }

        return self.post(method="exec", params=params)

    def execution_log(self, fortigate: str):
        """Retrieves the script execution log on a FortiGate.

        Args:
            fortigate (str): Name of the FortiGate.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": f"/dvmdb/script/log/list/device/{fortigate}"
        }

        return self.post(method="get", params=params)


from setuptools import setup, find_packages

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="pyfortimanager",
    version="1.0.8",
    description="Python API client library for Fortinet's FortiManager.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="Apache2",
    author="Rasmus Thing",
    author_email="rasmus.thing@bestseller.com",
    url="https://github.com/BESTSELLER/pyfortimanager",
    python_requires=">=3.8, <4",
    install_requires=[
        "requests>=2.20.0,<3.0"
    ],
    zip_safe=False,
    keywords=[
        "fortinet",
        "fortimanager"
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12"
    ]
)


from pyfortimanager.core.fortimanager import FortiManager


class System(FortiManager):
    """API class for the FortiManager system.
    """

    def __init__(self, **kwargs):
        super(System, self).__init__(**kwargs)
    
    def firmware(self, platform: str=None, product: str=None):
        """Retrieves all available firmware versions for all or a specific product line.

        Args:
            platform (str, optional): Model number of a specific product type. Ex. FortiAP-U432F.
            product (str, optional): A specific product group. Ex. FMG, FGT, FSW, FAP, etc.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/um/image/version/list",
            "data":
                {
                    "platform": platform,
                    "product": product
                }
        }

        return self.post(method="exec", params=params)

    def ha(self):
        """Obtain FortiManager HA information and status of the system.

        Returns:
            dict: JSON data.
        """
   
        params = {
            "url": "/sys/ha/status"
        }

        return self.post(method="get", params=params)
 
    def proxy(self, target: list, action: str="get", payload: object=None, resource: str=None, timeout: int=None):
        """Send and receive a JSON request to/from managed FortiGates. The response will be an array of data, one for each queried device.

        Args:
            target (list): A list of FortiGates with their ADOM to target. Ex. ["/adom/<name_of_adom>/device/<name_of_fortigate>"]
            action (str): Specify HTTP action for the request: get, post, put, delete. Default is get.
            resource (str): URL on the remote device to be accessed. Ex. /api/v2/<rest_of_the_endpoint>
            payload (object, optional): An object containing the payload needed for the resource URL. Ex. { "vdom": "root", "admin": "enable" }
            timeout (int, optional): How long to wait for the FortiGate to respond. Defaults to the proxy_timeout set when the API was instantiated.

        Returns:
            dict: JSON data.
        """

        params = {
            "url": "/sys/proxy/json",
            "data":
                {
                    "target": target,
                    "action": action,
                    "timeout": timeout or self.api.proxy_timeout,
                    "resource": resource
                }
        }

        # Optional fields
        if payload:
            params['data']['payload'] = payload

        return self.post(method="exec", params=params)

    def reboot(self, message: str=None):
        """Reboots the FortiManager.

        Args:
            message (str, optional): Optional message to be stored in the event log.

        Returns:
            dict: JSON data.
        """
   
        params = {
            "url": "/sys/reboot",
            "data": {
                "message": message
            }
        }

        return self.post(method="exec", params=params)

    def status(self):
        """Retrieves the FortiManager system status.

        Returns:
            dict: JSON data.
        """
   
        params = {
            "url": "/sys/status"
        }

        return self.post(method="get", params=params)

    def tasks(self, task:int=None, filter: list=None, loadsub: bool=True):
        """Retrieves all FortiManager tasks or a single task.

        Args:
            task (int): ID of a specific task.
            filter (list): Filter the result according to a set of criteria. example: List [ "{attribute}", "==", "{value}" ]
            loadsub (bool): Enable or disable the return of any sub-objects. Default is enabled.

        Returns:
            dict: JSON data.
        """
   
        params = {
            "url": "/task/task",
            "filter": filter,
            "loadsub": loadsub
        }

        # Retrieve a single task
        if task:
            params['url'] = f"/task/task/{task}/line"

        return self.post(method="get", params=params)